@@ -36,7 +36,7 @@ ShadowRenderer::ShadowRenderer(IrrlichtDevice *device, Client *client) :
 
 	m_shadow_map_max_distance = g_settings->getFloat("shadow_map_max_distance");
 
-	m_shadow_map_texture_size = g_settings->getFloat("shadow_map_texture_size");
+	m_shadow_map_texture_size = g_settings->getU32("shadow_map_texture_size");
 
 	m_shadow_map_texture_32bit = g_settings->getBool("shadow_map_texture_32bit");
 	m_shadow_map_colored = g_settings->getBool("shadow_map_color");
@@ -107,7 +107,7 @@ void ShadowRenderer::disable()
 void ShadowRenderer::preInit(IWritableShaderSource *shsrc)
 {
 	if (g_settings->getBool("enable_dynamic_shadows")) {
-		shsrc->addShaderConstantSetterFactory(new ShadowConstantSetterFactory());
+		shsrc->addShaderUniformSetterFactory(new ShadowUniformSetterFactory());
 	}
 }
 
@@ -177,14 +177,15 @@ void ShadowRenderer::removeNodeFromShadowList(scene::ISceneNode *node)
 	node->forEachMaterial([] (auto &mat) {
 		mat.setTexture(TEXTURE_LAYER_SHADOW, nullptr);
 	});
-	for (auto it = m_shadow_node_array.begin(); it != m_shadow_node_array.end();) {
-		if (it->node == node) {
-			it = m_shadow_node_array.erase(it);
-			break;
-		} else {
-			++it;
-		}
+
+	auto it = std::find(m_shadow_node_array.begin(), m_shadow_node_array.end(), node);
+	if (it == m_shadow_node_array.end()) {
+		infostream << "removeNodeFromShadowList: " << node << " not found" << std::endl;
+		return;
 	}
+	// swap with last, then remove
+	*it = m_shadow_node_array.back();
+	m_shadow_node_array.pop_back();
 }
 
 void ShadowRenderer::updateSMTextures()
@@ -254,17 +255,14 @@ void ShadowRenderer::updateSMTextures()
 	if (!m_shadow_node_array.empty()) {
 		bool reset_sm_texture = false;
 
-		// detect if SM should be regenerated
+		// clear texture if requested
 		for (DirectionalLight &light : m_light_list) {
-			if (light.should_update_map_shadow)
-				m_force_update_shadow_map = true;
+			reset_sm_texture |= light.should_update_map_shadow;
 			light.should_update_map_shadow = false;
 		}
 
-		if (m_force_update_shadow_map) {
+		if (reset_sm_texture || m_force_update_shadow_map)
 			m_current_frame = 0;
-			reset_sm_texture = true;
-		}
 
 		video::ITexture* shadowMapTargetTexture = shadowMapClientMapFuture;
 		if (shadowMapTargetTexture == nullptr)
@@ -273,20 +271,17 @@ void ShadowRenderer::updateSMTextures()
 		// Update SM incrementally:
 		for (DirectionalLight &light : m_light_list) {
 			// Static shader values.
-			for (auto cb : {m_shadow_depth_cb, m_shadow_depth_entity_cb, m_shadow_depth_trans_cb})
+			for (auto cb : {m_shadow_depth_cb, m_shadow_depth_entity_cb, m_shadow_depth_trans_cb}) {
 				if (cb) {
-					cb->MapRes = (f32)m_shadow_map_texture_size;
+					cb->MapRes = (u32)m_shadow_map_texture_size;
 					cb->MaxFar = (f32)m_shadow_map_max_distance * BS;
 					cb->PerspectiveBiasXY = getPerspectiveBiasXY();
 					cb->PerspectiveBiasZ = getPerspectiveBiasZ();
 					cb->CameraPos = light.getFuturePlayerPos();
 				}
+			}
 
-			// set the Render Target
-			// right now we can only render in usual RTT, not
-			// Depth texture is available in irrlicth maybe we
-			// should put some gl* fn here
-
+			// Note that force_update means we're drawing everything one go.
 
 			if (m_current_frame < m_map_shadow_update_frames || m_force_update_shadow_map) {
 				m_driver->setRenderTarget(shadowMapTargetTexture, reset_sm_texture, true,
@@ -415,7 +410,7 @@ video::ITexture *ShadowRenderer::getSMTexture(const std::string &shadow_map_name
 				shadow_map_name.c_str(), texture_format);
 	}
 
-	return m_driver->getTexture(shadow_map_name.c_str());
+	return m_driver->findTexture(shadow_map_name.c_str());
 }
 
 void ShadowRenderer::renderShadowMap(video::ITexture *target,
@@ -519,6 +514,9 @@ void ShadowRenderer::mixShadowsQuad()
  * Shaders system with custom IShaderConstantSetCallBack without messing up the
  * code too much. If anyone knows how to integrate this with the standard MT
  * shaders, please feel free to change it.
+ *
+ * TODO: as of now (2025) it should be possible to hook these up to the normal
+ * shader system.
  */
 
 void ShadowRenderer::createShaders()
@@ -691,21 +689,19 @@ std::string ShadowRenderer::readShaderFile(const std::string &path)
 
 ShadowRenderer *createShadowRenderer(IrrlichtDevice *device, Client *client)
 {
+	if (!g_settings->getBool("enable_dynamic_shadows"))
+		return nullptr;
+
 	// disable if unsupported
-	if (g_settings->getBool("enable_dynamic_shadows")) {
-		// See also checks in builtin/mainmenu/settings/dlg_settings.lua
-		const video::E_DRIVER_TYPE type = device->getVideoDriver()->getDriverType();
-		if (type != video::EDT_OPENGL && type != video::EDT_OPENGL3) {
-			warningstream << "Shadows: disabled dynamic shadows due to being unsupported" << std::endl;
-			g_settings->setBool("enable_dynamic_shadows", false);
-		}
+	// See also checks in builtin/mainmenu/settings/dlg_settings.lua
+	const video::E_DRIVER_TYPE type = device->getVideoDriver()->getDriverType();
+	if (type != video::EDT_OPENGL && type != video::EDT_OPENGL3) {
+		warningstream << "Shadows: disabled dynamic shadows due to being unsupported" << std::endl;
+		g_settings->setBool("enable_dynamic_shadows", false);
+		return nullptr;
 	}
 
-	if (g_settings->getBool("enable_dynamic_shadows")) {
-		ShadowRenderer *shadow_renderer = new ShadowRenderer(device, client);
-		shadow_renderer->initialize();
-		return shadow_renderer;
-	}
-
-	return nullptr;
+	ShadowRenderer *shadow_renderer = new ShadowRenderer(device, client);
+	shadow_renderer->initialize();
+	return shadow_renderer;
 }
