@@ -763,23 +763,23 @@ inline std::string str_join(const std::vector<std::string> &list,
 
 #if IS_CLIENT_BUILD
 /**
- * Create a UTF8 std::string from an irr::core::stringw.
+ * Create a UTF8 std::string from an core::stringw.
  */
 [[nodiscard]]
-inline std::string stringw_to_utf8(const irr::core::stringw &input)
+inline std::string stringw_to_utf8(const core::stringw &input)
 {
 	std::wstring_view sv(input.c_str(), input.size());
 	return wide_to_utf8(sv);
 }
 
 /**
- * Create an irr::core:stringw from a UTF8 std::string.
+ * Create an core:stringw from a UTF8 std::string.
  */
 [[nodiscard]]
-inline irr::core::stringw utf8_to_stringw(std::string_view input)
+inline core::stringw utf8_to_stringw(std::string_view input)
 {
 	std::wstring str = utf8_to_wide(input);
-	return irr::core::stringw(str.c_str(), str.size());
+	return core::stringw(str.c_str(), str.size());
 }
 #endif
 
